@@ -68,13 +68,16 @@ def find_port(pid=None, ctx=local_files.DEFAULT_CONTEXT):
 ####  ^^
 
 
-def __find_port__linux(pid=None, ctx=local_files.DEFAULT_CONTEXT):
+def __find_port__linux(pid=None, ctx=local_files.DEFAULT_CONTEXT,
+        retries_left=3):
     """This returns a valid port or raises a RuntimeError exception.  It never
     returns anything else."""
     if pid is None:
         pid = find_pid(start_if_not_running=True, ctx=ctx)
 
     if pid is None:
+        if retries_left:
+            return __find_port__linux(pid, ctx, retries_left-1)
         raise RuntimeError("Have no PID to use to look up port.")
 
     proc_dir = "/proc/%d" % (pid,)
@@ -92,6 +95,8 @@ def __find_port__linux(pid=None, ctx=local_files.DEFAULT_CONTEXT):
                         (dirent_path,))
                 continue
     except OSError:
+        if retries_left:
+            return __find_port__linux(pid, ctx, retries_left-1)
         logging.exception("Unable to find file descriptors in %s" % proc_dir)
         raise RuntimeError("Unable to find file descriptors in %s" % proc_dir)
 
@@ -129,6 +134,8 @@ def __find_port__linux(pid=None, ctx=local_files.DEFAULT_CONTEXT):
                 break
 
     if port is None:
+        if retries_left:
+            return __find_port__linux(pid, ctx, retries_left-1)
         raise RuntimeError("Unable to find listening port")
 
     return port
