@@ -208,7 +208,7 @@ class Context():
     def __init__(self, run_dir, db_dir, config_dir, with_auth=True,
             keyring=gnomekeyring):  # (cache, data, config)
 
-        self.couchdb_log_level = 'info'
+        self.couchdb_log_level = 'notice'
 
         for d in (run_dir, db_dir, config_dir):
             if not os.path.isdir(d):
