@@ -212,7 +212,10 @@ def _get_management_data(record_type, key, uri=None,
         if key in record.value:  # EAFP, rather than LBYL?  Nones default?
             value = record.value[key]
             if value is not None:
-                values.append(value)
+                if "_order" in value:  # MergableList, so decode.
+                    values.append([value[k] for k in value["_order"]])
+                else:
+                    values.append(value)
             else:
                 logging.debug("skipping record empty %s", key)
         else:
