@@ -23,6 +23,7 @@ import desktopcouch.tests as test_environment
 from desktopcouch.pair.couchdb_pairing import couchdb_io
 from desktopcouch.records.server import CouchDatabase
 from desktopcouch.records.record import Record
+from desktopcouch.replication_services import ubuntuone
 from twisted.trial import unittest
 import uuid
 import os
@@ -39,6 +40,9 @@ class TestCouchdbIo(unittest.TestCase):
         self.foo_database = CouchDatabase('foo', create=True, uri=URI,
                 ctx=test_environment.test_context)
         #create some records to pull out and test
+        self.foo_database = CouchDatabase('bar', create=True, uri=URI,
+                ctx=test_environment.test_context)
+        #create some records to pull out and test
         self.foo_database.put_record(Record({
             "key1_1": "val1_1", "key1_2": "val1_2", "key1_3": "val1_3",
             "record_type": "test.com"}))
@@ -49,6 +53,15 @@ class TestCouchdbIo(unittest.TestCase):
             "key13_1": "va31_1", "key3_2": "val3_2", "key3_3": "val3_3",
             "record_type": "test.com"}))
 
+        couchdb_io.put_static_paired_service({ 
+            "consumer_key": str("abcdef"),
+            "consumer_secret": str("ghighjklm"),
+            "token": str("opqrst"),
+            "token_secret": str("uvwxyz")}, "ubuntuone", uri=URI, 
+            ctx=test_environment.test_context)
+        excl = ubuntuone.ReplicationExclusion(ctx=test_environment.test_context)
+        excl.exclude("bar")
+
     def tearDown(self):
         """tear down each test"""
         del self.mgt_database._server['management']
@@ -95,7 +108,7 @@ class TestCouchdbIo(unittest.TestCase):
                 uri=URI, ctx=test_environment.test_context)
 
         pairings = list(couchdb_io.get_pairings(ctx=test_environment.test_context))
-        self.assertEqual(3, len(pairings))
+        self.assertEqual(4, len(pairings))  #  3, plus 1 from setUp()
         self.assertEqual(pairings[0].value["oauth"], oauth_data)
         self.assertEqual(pairings[0].value["server"], hostname)
         self.assertEqual(pairings[0].value["pairing_identifier"], remote_uuid)
@@ -126,6 +139,7 @@ class TestCouchdbIo(unittest.TestCase):
         names = couchdb_io.get_database_names_replicatable(uri=URI, ctx=test_environment.test_context)
         self.assertFalse('management' in names)
         self.assertTrue('foo' in names)
+        self.assertFalse('bar' in names, names)  # is excluded
 
     def test_get_my_host_unique_id(self):
         got = couchdb_io.get_my_host_unique_id(uri=URI, ctx=test_environment.test_context)
