@@ -125,6 +125,7 @@ class RecordData(object):
         validate(data)
         self._data = data
         self._attachments = dict()
+        self._detached = set()
 
     def __getitem__(self, key):
         value = self._data[key]
@@ -169,6 +170,7 @@ class RecordData(object):
         """Remove a BLOB attached to a document."""
         try:
             self._attachments.pop(filename)
+            self._detached.add(filename)
         except KeyError:
             raise KeyError("%r is not attached to this document" % (filename,))
 
