@@ -23,6 +23,7 @@
 
 import httplib2, urlparse, cgi, copy
 from time import time
+import socket
 
 # please keep desktopcouch python 2.5 compatible for now
 
@@ -42,6 +43,7 @@ from couchdb import Server
 from couchdb.client import ResourceNotFound, ResourceConflict, uri as couchdburi
 from couchdb.design import ViewDefinition
 from record import Record
+import logging
 
 #DEFAULT_DESIGN_DOCUMENT = "design"
 DEFAULT_DESIGN_DOCUMENT = None  # each view in its own eponymous design doc.
@@ -142,29 +144,59 @@ class CouchDatabaseBase(object):
 
     def __init__(self, database, uri, record_factory=None, create=False,
                  server_class=Server, **server_class_extras):
-        self.server_uri = uri
-        self._server = server_class(self.server_uri, **server_class_extras)
-        if database not in self._server:
-            if create:
-                self._server.create(database)
-            else:
-                raise NoSuchDatabase(database)
-        self.db = self._server[database]
+        self._database_name = database
+        self._create = create
+        self._server_class = server_class
+        self._server_class_extras = server_class_extras
         self.record_factory = record_factory or Record
+        self.server_uri = uri
+        self._reconnect()
         self._changes_since = self.db.info()["update_seq"]
         self._changes_last_used = 0  # Immediate run works.
 
+    @staticmethod
+    def _is_bug_lp539674(ex):
+        return isinstance(ex, AttributeError) and \
+                ex.args == ("'NoneType' object has no attribute 'makefile'",)
+
+    def with_reconnects(self, func, *args, **kwargs):
+        for retry in (2, 1, None):
+            try:
+                return func(*args, **kwargs)
+            except Exception, e:
+                if self._is_bug_lp539674(e) and retry:
+                    logging.warn("DB connection timed out.  Reconnecting.")
+                    self._reconnect()
+                    continue
+                elif isinstance(e, socket.error):
+                    logging.warn("Other socket error %s.  Reconnecting.", e)
+                    time.sleep(0.3)  
+                    self._reconnect()
+                    continue
+                else:
+                    raise
+
+    def _reconnect(self):
+        logging.info("Connecting to %s." % (self.server_uri or "discovered local port",))
+        self._server = self._server_class(self.server_uri,
+                **self._server_class_extras)
+        if self._database_name not in self._server:
+            if self._create:
+                self._server.create(self._database_name)
+            else:
+                raise NoSuchDatabase(self._database_name)
+        self.db = self._server[self._database_name]
 
     def _temporary_query(self, map_fun, reduce_fun=None, language='javascript',
             wrapper=None, **options):
         """Pass-through to CouchDB library.  Deprecated."""
-        return self.db.query(map_fun, reduce_fun, language,
+        return self.with_reconnects(self.db.query, map_fun, reduce_fun, language,
               wrapper, **options)
 
     def get_record(self, record_id):
         """Get a record from back end storage."""
         try:
-            couch_record = self.db[record_id]
+            couch_record = self.with_reconnects(self.db.__getitem__, record_id)
         except ResourceNotFound:
             return None
         data = {}
@@ -193,11 +225,16 @@ class CouchDatabaseBase(object):
             # Do not rely on couchdb to create an ID for us.
             from uuid import uuid4
             record.record_id = uuid4().hex
-        self.db[record.record_id] = record._data
+        self.with_reconnects(self.db.__setitem__,
+                record.record_id, record._data)
+
+        for attachment_name in getattr(record, "_detached", []):
+            self.with_reconnects(self.db.delete_attachment,
+                record._data, attachment_name)
 
         for attachment_name in record.list_attachments():
             data, content_type = record.attachment_data(attachment_name)
-            self.db.put_attachment(
+            self.with_reconnects(self.db.put_attachment,
                 record._data, data, attachment_name, content_type)
 
         return record.record_id
@@ -215,16 +252,22 @@ class CouchDatabaseBase(object):
         # although with a single record we need to test for the
         # revisison, with a batch we do not, but we have to make sure
         # that we did not get an error
-        batch_put_result = self.db.update([record._data for record in batch])
+        batch_put_result = self.with_reconnects(
+                self.db.update, [record._data for record in batch])
         for current_tuple in batch_put_result:
             success, docid, rev_or_exc = current_tuple
             if success:
                 record = records_hash[docid]
                 # set the new rev
                 record._data["_rev"] = rev_or_exc
+
+                for attachment_name in getattr(record, "_detached", []):
+                    self.with_reconnects(self.db.delete_attachment,
+                        record._data, attachment_name)
+
                 for attachment_name in record.list_attachments():
                     data, content_type = record.attachment_data(attachment_name)
-                    self.db.put_attachment(
+                    self.with_reconnects(self.db.put_attachment,
                         {"_id":record.record_id, "_rev":record["_rev"]},
                         data, attachment_name, content_type)
         # all success record have the blobs added we return result of
@@ -249,7 +292,8 @@ class CouchDatabaseBase(object):
         # as far as we know. (If they're not, we'll get a
         # ResourceConflict later on, from which we can recover.)
         if cached_record is None:
-            cached_record = self.db[record_id]
+            cached_record = self.with_reconnects(self.db.__getitem__,
+                    record_id)
         if isinstance(cached_record, Record):
             cached_record = cached_record._data
         record = copy.deepcopy(cached_record)
@@ -294,13 +338,15 @@ class CouchDatabaseBase(object):
             # it.
             if modified:
                 try:
-                    self.db[record_id] = record
+                    self.with_reconnects(self.db.__setitem__, record_id,
+                            record)
                 except ResourceConflict:
                     # We got a conflict, meaning the record has
                     # changed in the database since we last loaded it
                     # into memory. Let's get a fresh copy and try
                     # again.
-                    record = self.db[record_id]
+                    record = self.with_reconnects(self.db.__getitem__,
+                            record_id)
                     continue
             # If we get here, nothing remains to be done, and we can
             # take a well deserved break.
@@ -308,17 +354,17 @@ class CouchDatabaseBase(object):
 
     def delete_record(self, record_id):
         """Delete record with given id"""
-        record = self.db[record_id]
+        record = self.with_reconnects(self.db.__getitem__, record_id)
         record.setdefault('application_annotations', {}).setdefault(
             'Ubuntu One', {}).setdefault('private_application_annotations', {})[
             'deleted'] = True
-        self.db[record_id] = record
+        self.with_reconnects(self.db.__setitem__, record_id, record)
 
     def record_exists(self, record_id):
         """Check if record with given id exists."""
         if record_id not in self.db:
             return False
-        record = self.db[record_id]
+        record = self.with_reconnects(self.db.__getitem__, record_id)
         return not row_is_deleted(record)
 
     def delete_view(self, view_name, design_doc=DEFAULT_DESIGN_DOCUMENT):
@@ -332,7 +378,8 @@ class CouchDatabaseBase(object):
         # No atomic updates.  Only read & mutate & write.  Le sigh.
         # First, get current contents.
         try:
-            view_container = self.db[doc_id]["views"]
+            view_container = self.with_reconnects(
+                    self.db.__getitem__, doc_id)["views"]
         except (KeyError, ResourceNotFound):
             raise KeyError
 
@@ -358,7 +405,7 @@ class CouchDatabaseBase(object):
 
             # Remove design document.  This assumes there are only views in
             # design documents.  :(
-            del self.db[doc_id]
+            self.with_reconnects(self.db.__delitem__, doc_id)
 
         assert not self.view_exists(view_name, design_doc)
 
@@ -371,7 +418,8 @@ class CouchDatabaseBase(object):
             design_doc = view_name
 
         view_id_fmt = "_design/%(design_doc)s/_view/%(view_name)s"
-        return self.db.view(view_id_fmt % locals(), **params)
+        return self.with_reconnects(self.db.view, view_id_fmt % locals(),
+                **params)
 
     def add_view(self, view_name, map_js, reduce_js,
             design_doc=DEFAULT_DESIGN_DOCUMENT):
@@ -393,7 +441,8 @@ class CouchDatabaseBase(object):
         doc_id = "_design/%(design_doc)s" % locals()
 
         try:
-            view_container = self.db[doc_id]["views"]
+            view_container = \
+                    self.with_reconnects(self.db.__getitem__, doc_id)["views"]
             return view_name in view_container
         except (KeyError, ResourceNotFound):
             return False
@@ -404,7 +453,7 @@ class CouchDatabaseBase(object):
         in it."""
         doc_id = "_design/%(design_doc)s" % locals()
         try:
-            return list(self.db[doc_id]["views"])
+            return list(self.with_reconnects(self.db.__getitem__, doc_id)["views"])
         except (KeyError, ResourceNotFound):
             return []
 
