@@ -346,6 +346,9 @@ class TestCouchDatabase(testtools.TestCase):
         constructed_record.attach(content, "nu/mbe/rs", "text/plain")
         constructed_record.attach("string", "another document", "text/plain")
 
+        constructed_record.attach("XXXXXXXXX", "never used", "text/plain")
+        constructed_record.detach("never used")  # detach works before commit.
+
         # We can read from a document that we constructed.
         out_file, out_content_type = \
                 constructed_record.attachment_data("nu/mbe/rs")
@@ -365,8 +368,21 @@ class TestCouchDatabase(testtools.TestCase):
         # get new
         retrieved_record = self.database.get_record(record_id)
 
-        # To replace, one must remove first.
+        # Remove and test that it is indeed gone.
         retrieved_record.detach("another document")
+        self.assertRaises(KeyError, retrieved_record.detach,
+                "another document")
+
+        # push new version
+        record_id = self.database.put_record(retrieved_record)
+        # get new
+        retrieved_record = self.database.get_record(record_id)
+
+        # We can get a list of attachments.
+        self.assertEqual(set(retrieved_record.list_attachments()),
+                set(["nu/mbe/rs", "Document"]))
+
+        # Send back name.
         retrieved_record.attach(content, "another document", "text/plain")
 
         # push new version
